USE [BillingData]
GO

/****** 	
Object:			StoredProcedure [dbo].[usp_GetBillingData_CASE]    
Script Date:	04/15/2009 22:10:49 
Description:	The methodology below uses CASE WHEN to 
				report Monthly Billing Data

				This following USP's are identical:
				usp_GetBillingData
				usp_GetBillingData_CASE
				usp_GetBillingData_COALESCE
				usp_GetBillingData_PIVOT
				
				Each produces the same results using different
				methods.
******/

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO


CREATE PROCEDURE [dbo].[usp_GetBillingData_CASE]
AS

BEGIN
SET NOCOUNT ON;

--Alternative Solution*********************************************************************Begin
--DECLARE	@YEAR 		VARCHAR(4), 
--		    @MONTH		VARCHAR(2), 
--		    @ISDATE		VARCHAR(6)

--SELECT 	@YEAR 		= CAST(YEAR(GETDATE()) AS INT), 
--		    @MONTH 		= CAST(MONTH(GETDATE()) AS INT)		current month
--		    @MONTH 		= CAST(MONTH(GETDATE()) - 1 AS INT) previous month

--SET		@ISDATE		= @YEAR + RIGHT('00' + @MONTH,2)
--Alternative Solution*********************************************************************End

DECLARE	@SQL		VARCHAR(5000), 
		@ISYear		VARCHAR(4),
		@ISMonth	VARCHAR(2),
		@ISDate		VARCHAR(6)
		
SELECT	@ISYear	= LEFT(ReportDate, 4)
FROM	dbo.Report_Date

SELECT	@ISDate = ReportDate
FROM	dbo.Report_Date

--This derived query summarizes the data
SELECT	Customer, 
		SUM([JAN])					AS [JAN], 
		SUM([FEB])					AS [FEB], 
		SUM([MAR])					AS [MAR], 
		SUM([APR])					AS [APR], 
		SUM([MAY])					AS [MAY], 
		SUM([JUN])					AS [JUN], 
		SUM([JUL])					AS [JUL], 
		SUM([AUG])					AS [AUG], 
		SUM([SEP])					AS [SEP], 
		SUM([OCT])					AS [OCT], 
		SUM([NOV])					AS [NOV], 
		SUM([DEC])					AS [DEC],  
		SUM(Total)					AS Total
FROM	(
--This derived query formats the data
SELECT	Customer, 
		CASE WHEN BillYrMo = @ISYear + '01'	THEN SUM(BillAmount) ELSE 0 END	AS [JAN], 
		CASE WHEN BillYrMo = @ISYear + '02'	THEN SUM(BillAmount) ELSE 0 END	AS [FEB], 
		CASE WHEN BillYrMo = @ISYear + '03'	THEN SUM(BillAmount) ELSE 0 END	AS [MAR],  
		CASE WHEN BillYrMo = @ISYear + '04'	THEN SUM(BillAmount) ELSE 0 END	AS [APR],  
		CASE WHEN BillYrMo = @ISYear + '05'	THEN SUM(BillAmount) ELSE 0 END	AS [MAY],  
		CASE WHEN BillYrMo = @ISYear + '06'	THEN SUM(BillAmount) ELSE 0 END	AS [JUN],  
		CASE WHEN BillYrMo = @ISYear + '07'	THEN SUM(BillAmount) ELSE 0 END	AS [JUL],  
		CASE WHEN BillYrMo = @ISYear + '08'	THEN SUM(BillAmount) ELSE 0 END	AS [AUG],  
		CASE WHEN BillYrMo = @ISYear + '09'	THEN SUM(BillAmount) ELSE 0 END	AS [SEP],  
		CASE WHEN BillYrMo = @ISYear + '10'	THEN SUM(BillAmount) ELSE 0 END	AS [OCT],  
		CASE WHEN BillYrMo = @ISYear + '11'	THEN SUM(BillAmount) ELSE 0 END	AS [NOV],  
		CASE WHEN BillYrMo = @ISYear + '12'	THEN SUM(BillAmount) ELSE 0 END	AS [DEC], 
		CASE WHEN BillYrMo <> ''			THEN SUM(BillAmount) ELSE 0 END AS Total 
FROM	(
SELECT	Customer, 
        CAST(YEAR(BillDate) AS VARCHAR(4)) + 
			RIGHT('0' + CAST(MONTH(BillDate) AS VARCHAR(4)),2)	AS BillYrMo, 
        CAST(YEAR(BillDate) AS VARCHAR(4))						AS BillYr, 
		BillDate, 
		SUM(BillAmount)											AS BillAmount
FROM	dbo.Billing
WHERE	CAST(YEAR(BillDate) AS VARCHAR(4)) + 
			RIGHT('0' + CAST(MONTH(BillDate) AS VARCHAR(4)),2)	<= @ISDate
GROUP BY
		Customer,
		BillDate
		) MICHAELDNEWBY
GROUP BY
		Customer,
		BillYrMo
		) MICHAELDNEWBY2
GROUP BY
		Customer

UNION ALL

SELECT	Customer, 
		SUM([JAN])					AS [JAN], 
		SUM([FEB])					AS [FEB], 
		SUM([MAR])					AS [MAR], 
		SUM([APR])					AS [APR], 
		SUM([MAY])					AS [MAY], 
		SUM([JUN])					AS [JUN], 
		SUM([JUL])					AS [JUL], 
		SUM([AUG])					AS [AUG], 
		SUM([SEP])					AS [SEP], 
		SUM([OCT])					AS [OCT], 
		SUM([NOV])					AS [NOV], 
		SUM([DEC])					AS [DEC],  
		SUM(Total)					AS Total
FROM	(
--This derived query formats the data
SELECT	'_All Customers'														AS Customer, 
		CASE WHEN BillYrMo = @ISYear + '01'	THEN SUM(BillAmount) ELSE 0 END	AS [JAN], 
		CASE WHEN BillYrMo = @ISYear + '02'	THEN SUM(BillAmount) ELSE 0 END	AS [FEB], 
		CASE WHEN BillYrMo = @ISYear + '03'	THEN SUM(BillAmount) ELSE 0 END	AS [MAR],  
		CASE WHEN BillYrMo = @ISYear + '04'	THEN SUM(BillAmount) ELSE 0 END	AS [APR],  
		CASE WHEN BillYrMo = @ISYear + '05'	THEN SUM(BillAmount) ELSE 0 END	AS [MAY],  
		CASE WHEN BillYrMo = @ISYear + '06'	THEN SUM(BillAmount) ELSE 0 END	AS [JUN],  
		CASE WHEN BillYrMo = @ISYear + '07'	THEN SUM(BillAmount) ELSE 0 END	AS [JUL],  
		CASE WHEN BillYrMo = @ISYear + '08'	THEN SUM(BillAmount) ELSE 0 END	AS [AUG],  
		CASE WHEN BillYrMo = @ISYear + '09'	THEN SUM(BillAmount) ELSE 0 END	AS [SEP],  
		CASE WHEN BillYrMo = @ISYear + '10'	THEN SUM(BillAmount) ELSE 0 END	AS [OCT],  
		CASE WHEN BillYrMo = @ISYear + '11'	THEN SUM(BillAmount) ELSE 0 END	AS [NOV],  
		CASE WHEN BillYrMo = @ISYear + '12'	THEN SUM(BillAmount) ELSE 0 END	AS [DEC], 
		CASE WHEN BillYrMo <> ''			THEN SUM(BillAmount) ELSE 0 END AS Total 
FROM	(
SELECT	Customer, 
        CAST(YEAR(BillDate) AS VARCHAR(4)) + 
			RIGHT('0' + CAST(MONTH(BillDate) AS VARCHAR(4)),2)	AS BillYrMo, 
        CAST(YEAR(BillDate) AS VARCHAR(4))						AS BillYr, 
		BillDate, 
		SUM(BillAmount)											AS BillAmount
FROM	dbo.Billing
WHERE	CAST(YEAR(BillDate) AS VARCHAR(4)) + 
			RIGHT('0' + CAST(MONTH(BillDate) AS VARCHAR(4)),2)	<= @ISDate
GROUP BY
		Customer,
		BillDate
		) MICHAELDNEWBY
GROUP BY
		Customer,
		BillYrMo
		) MICHAELDNEWBY2
GROUP BY
		Customer
ORDER BY
		Customer

END

GO